
      subroutine wlic1d4md(f,u,n1,dt,dx,mesh_n,idr,idm)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     THINC/WLIC sample code version 1.
!     Author: Kensuke YOKOI
!     Email: kensuke@kensuke.biz
!     Date: Sept. 10, 2006  
!     
!     Dimensional splitting THINC/WLIC 1D solver for multi-dimension. 
!     This subroutine works for both of 2D and 3D cases and probably
!     for more than 3D cases as well.
!
!     f(i): vof function at (i)
!     u(i): idr(x, y or z)-component of the velocity at (i-1/2)
!     n1(i,1): x-component of the unit normal at (i),
!             n1(i,2): y-, n1(i,3): z-
!     dt:
!     dx:
!     mesh_n: mesh number of idr direction
!     idr: direction of flux, 1:x, 2:y, 3:z
!     idm: spatial dimension 2 or 3, or more than 3(?)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      implicit none
      integer mesh_n            
      integer idr               
      integer idm               
      real*8 f(mesh_n)          
      real*8 u(mesh_n)          
      real*8 n1(mesh_n,idm)     
      real*8 dt,dx
      
      real*8 flux(mesh_n)       ! flux(i): flux at (i-1/2)
      integer i,j,isgn,ib,ii
      real*8 nn                 ! denominator of weights
      real*8 alpha,beta
      real*8 xc,a1,a3,a4,a5
      real*8 weight             ! weight for idr direction interface
      
      beta = 3.5d0
      do i=1,mesh_n
         flux(i) = 0.0d0
      end do
      
      do i=2,mesh_n-1
         if(u(i).le.0.0d0) then 
            isgn=0
            ii = i
         else
            isgn=1
            ii = i-1
         end if

         nn = 0.0d0
         do j=1,idm ! compute the denominator of weights
            nn = nn + dabs(n1(ii,j))
         end do
         
         if((f(ii).ge.1.0d0-1.0d-10) ! If f ~ 0 or 1, or nn ~ 0,
     1        .or.(f(ii).le.1.0d-10)) then ! simple upwind method is used.
            flux(i) = f(ii)*u(i)*dt
         else if(nn.le.1.0d-10) then ! to avoid zero denominator
            flux(i) = f(ii)*u(i)*dt
         else
            ib=max(1,ii-1)      ! for a boundary of domain
            if(f(ib).le.f(ii+1)) then 
               alpha = 1.0d0    ! to determine the direction of interface
            else 
               alpha = -1.0d0 
            end if
            a1 = dexp(beta*(2.0*f(ii)-1.0d0)/alpha)
            a3 = dexp(beta)
            xc = 0.5d0/beta*dlog((a3**2-a1*a3)/(a1*a3-1.0d0))
            a4 = dcosh(beta*(dble(isgn)-u(i)*dt/dx-xc))
            a5 = dcosh(beta*(dble(isgn)-xc))
            flux(i) =0.5d0*(u(i)*dt-alpha*dx/beta*dlog(a4/a5))
            
            weight = dabs(n1(ii,idr))/nn ! weight for idr direction
            flux(i)=flux(i)*weight+f(ii)*u(i)*dt*(1.0d0-weight)
         end if
      end do
      
      flux(1)=0                 ! boundary conditon
      flux(mesh_n)=0
      
      do i=2, mesh_n-1
         f(i) = f(i)-(flux(i+1)-flux(i))/dx
      end do
      
      f(1)=f(2)                 ! boundary conditon
      f(mesh_n)=f(mesh_n-1)
      
      return
      end

